@@ -49,13 +49,9 @@ class PytorchModelCheckpointAndUpload(pl.callbacks.ModelCheckpoint):
 
 def PytorchCheckpointCallback(snapshot_interval,
                               snapshot_path,
-                              checkpoint_prefix,
-                              save_h5=False):
-
-    assert isinstance(save_h5, bool)
+                              checkpoint_prefix):
 
     os.makedirs(snapshot_path, exist_ok=True)
-
     checkpoint_name = os.path.join(snapshot_path,
                                    str(checkpoint_prefix) + '_{epoch:02d}')
 
@@ -156,15 +152,21 @@ class LightningCallback(pl.callbacks.base.Callback):
         self.num_step += 1
         logs = trainer.logger_connector.callback_metrics
 
-        sys_data = gpu_metrics()
-        sys_data.update(system_metrics())
+        # sys_data = gpu_metrics()
+        # Removing system_metrics for now, as these are not frequently used
+        # sys_data.update(system_metrics())
         if self.step_logging and self.num_step % self.log_evry_n_step == 0:
+            # try_mlflow_log(
+            #     log_metrics,
+            #     sys_data,
+            #     step=self.num_step,
+            #     epoch=self.current_epoch,
+            #     tags={'sys_metric': 'yes'})
             try_mlflow_log(
                 log_metrics,
-                sys_data,
-                step=self.num_step,
-                tags={'sys_metric': 'yes'})
-            try_mlflow_log(log_metrics, logs, step=self.num_step)
+                logs,
+                epoch=self.current_epoch,
+                step=self.num_step)
 
     def on_test_batch_end(self, trainer, pl_module, outputs, batch, batch_idx,
                           dataloader_idx):
@@ -183,14 +185,19 @@ class LightningCallback(pl.callbacks.base.Callback):
         logs = trainer.logger_connector.callback_metrics
 
         if self.step_logging and self.num_test_step % self.log_evry_n_step == 0:  # noqa: E501
-            sys_data = gpu_metrics()
-            sys_data.update(system_metrics())
+            # sys_data = gpu_metrics()
+            # sys_data.update(system_metrics())
+            # try_mlflow_log(
+            #     log_metrics,
+            #     sys_data,
+            #     step=self.num_step,
+            #     epoch=self.current_epoch,
+            #     tags={'sys_metric': 'yes'})
             try_mlflow_log(
                 log_metrics,
-                sys_data,
+                logs,
                 step=self.num_step,
-                tags={'sys_metric': 'yes'})
-            try_mlflow_log(log_metrics, logs, step=self.num_step)
+                epoch=self.current_epoch,)
 
     def on_test_end(self, trainer, pl_module):
         """Summary.
@@ -203,13 +210,16 @@ class LightningCallback(pl.callbacks.base.Callback):
             TYPE: Description
         """
         logs = trainer.logger_connector.callback_metrics
-        try_mlflow_log(log_metrics, logs, step=self.num_step)
+        try_mlflow_log(
+            log_metrics,
+            logs,
+            step=self.num_step)
 
     def on_test_start(self, trainer, pl_module):
         self.num_test_step = 0
 
-    # def on_test_batch_end(self, batch, logs=None):
-    #     self.test_step = batch
+    # def on_epoch_start(self, trainer, pl_module):
+    #     pass
 
     def on_epoch_end(self, trainer, pl_module):
         """Summary.
@@ -223,13 +233,20 @@ class LightningCallback(pl.callbacks.base.Callback):
         """
         # self.current_epoch = epoch
         logs = trainer.logger_connector.callback_metrics
+        self.current_epoch += 1
 
-        sys_data = gpu_metrics()
-        sys_data.update(system_metrics())
+        # sys_data = gpu_metrics()
+        # Removing system_metrics for now, as these are not frequently used
+        # sys_data.update(system_metrics())
+        # try_mlflow_log(
+        #     log_metrics,
+        #     sys_data,
+        #     step=self.num_step,
+        #     epoch=self.current_epoch,
+        #     tags={'sys_metric': 'yes'})
         try_mlflow_log(
             log_metrics,
-            sys_data,
+            logs,
             step=self.num_step,
-            tags={'sys_metric': 'yes'})
-        try_mlflow_log(log_metrics, logs, step=self.num_step)
+            epoch=self.current_epoch)
         # print('end of epoch')
