@@ -20,21 +20,21 @@ import time
 from google.protobuf.struct_pb2 import Struct
 from tempfile import gettempdir
 
-from segmind.data.converters.object_detection import (coco_to_voc_bbox,
-                                                      yolo_to_voc_bbox)
 from segmind.entities import Metric, Param, Run, RunStatus, RunTag, ViewType
 from segmind.entities.lifecycle_stage import LifecycleStage
 from segmind.exceptions import MlflowException
 from segmind.lite_extensions.client_utils import (_EXPERIMENT_ID_ENV_VAR,
                                                   _RUN_ID_ENV_VAR,
                                                   _get_experiment_id,
-                                                  _runid_exists)
-# from segmind.tracking import artifact_utils
+                                                  _runid_exists,
+                                                  get_token)
+from segmind.protos.service_lite_pb2 import RunTag as RunTagProto
 from segmind.tracking.client import MlflowClient
 from segmind.tracking.context import registry as context_registry
 from segmind.utils import env
 from segmind.utils.mlflow_tags import MLFLOW_PARENT_RUN_ID, MLFLOW_RUN_NAME
 from segmind.utils.validation import _validate_run_id
+from segmind.data.public import upload as upload_data
 
 # _EXPERIMENT_ID_ENV_VAR = "MLFLOW_EXPERIMENT_ID"
 # _EXPERIMENT_NAME_ENV_VAR = 'MLFLOW_EXPERIMENT_NAME'
@@ -235,6 +235,17 @@ def start_run(run_name=None, nested=False):
     return _active_run_stack[-1]
 
 
+def get_experiment_run(experiment_id=None, run_id=None, create_run_if_not_exists=False):
+    experiment_obj = MlflowClient().get_experiment(experiment_id=experiment_id)
+    os.environ[_EXPERIMENT_ID_ENV_VAR] = experiment_obj.experiment_id
+    if not run_id:
+        pass
+    else:
+        run_obj = MlflowClient().get_run(run_id=run_id)
+
+    return experiment_obj, run_obj
+
+
 def end_run(status=RunStatus.to_string(RunStatus.FINISHED)):
     """End an active MLflow run (if there is one).
 
@@ -293,16 +304,17 @@ def get_run(run_id):
     return MlflowClient().get_run(run_id)
 
 
-def log_param(key, value):
-    """Log a parameter under the current run. If no run is active, this method
-    will create a new active run.
+def _get_or_start_run():
+    """Summary.
 
-    Args:
-        key (str: Parameter name (string)
-        value (str): Parameter value (string, but will be string-ified if not)
+    Returns:
+        TYPE: Description
     """
-    run_id = _get_or_start_run().info.run_id
-    MlflowClient().log_param(run_id, key, value)
+    if len(_active_run_stack) > 0:
+        return _active_run_stack[-1]
+
+    return start_run()
+
 
 
 def set_tag(key, value):
@@ -328,7 +340,7 @@ def delete_tag(key):
     MlflowClient().delete_tag(run_id, key)
 
 
-def log_metric(key, value, step=None):
+def log_metric(key, value, step=None, epoch=None, tags={'sys_metric': 'no'}):
     """Log a metric under the current run. If no run is active, this method
     will create a new active run.
 
@@ -341,12 +353,50 @@ def log_metric(key, value, step=None):
         step (int, optional): Metric step (int). Defaults to zero if
                             unspecified.
     """
+    tags_arr = [RunTagProto(
+        key=key,
+        value=str(value)) for key, value in tags.items()]
     run_id = _get_or_start_run().info.run_id
     MlflowClient().log_metric(run_id, key, value, int(time.time() * 1000), step
-                              or 0)
+                              or 0, epoch=epoch or 0, tags=tags_arr)
 
 
-def log_metrics(metrics, step=None):
+def log_param(key, value, tags={'sys_param': 'no'}):
+    """Log a parameter under the current run. If no run is active, this method
+    will create a new active run.
+
+    Args:
+        key (str: Parameter name (string)
+        value (str): Parameter value (string, but will be string-ified if not)
+    """
+    tags_arr = [RunTagProto(
+        key=key,
+        value=str(value)) for key, value in tags.items()]
+    run_id = _get_or_start_run().info.run_id
+    MlflowClient().log_param(run_id, key, value, tags=tags_arr)
+
+
+def log_params(params, tags={'sys_param': 'no'}):
+    """Log a batch of params for the current run. If no run is active, this
+    method will create a new active run.
+
+    Args:
+        params (dict): Dictionary of param_name: String -> value: (String, but
+                    will be string-ified if not)
+    """
+    run_id = _get_or_start_run().info.run_id
+    tags_arr = [RunTagProto(
+        key=key,
+        value=str(value)) for key, value in tags.items()]
+    params_arr = [Param(
+        key,
+        str(value),
+        tags=tags_arr) for key, value in params.items()]
+    MlflowClient().log_batch(
+        run_id=run_id, metrics=[], params=params_arr, tags=tags_arr)
+
+
+def log_metrics(metrics, step=None, epoch=0, tags={'sys_metric': 'no'}):
     """Log multiple metrics for the current run. If no run is active, this
     method will create a new active run.
 
@@ -362,26 +412,46 @@ def log_metrics(metrics, step=None):
     """
     run_id = _get_or_start_run().info.run_id
     timestamp = int(time.time() * 1000)
+    tags_arr = [RunTagProto(
+        key=key,
+        value=str(value)) for key, value in tags.items()]
     metrics_arr = [
-        Metric(key, value, timestamp, step or 0)
+        Metric(
+            key,
+            value,
+            timestamp,
+            step or 0,
+            epoch=epoch or 0,
+            tags=tags_arr)
         for key, value in metrics.items()
     ]
     MlflowClient().log_batch(
-        run_id=run_id, metrics=metrics_arr, params=[], tags=[])
+        run_id=run_id, metrics=metrics_arr, params=[], tags=tags_arr)
 
 
-def log_params(params):
-    """Log a batch of params for the current run. If no run is active, this
-    method will create a new active run.
+def log_batch(metrics={}, params={}, tags={}, step=None, epoch=None):
+    """Summary.
 
     Args:
-        params (dict): Dictionary of param_name: String -> value: (String, but
-                    will be string-ified if not)
+        metrics (dict, optional): Description
+        params (dict, optional): Description
+        tags (dict, optional): Description
+        step (None, optional): Description
     """
     run_id = _get_or_start_run().info.run_id
+    tags_arr = [RunTagProto(
+        key=key,
+        value=str(value)) for key, value in tags.items()]
     params_arr = [Param(key, str(value)) for key, value in params.items()]
+
+    timestamp = int(time.time() * 1000)
+    metrics_arr = [
+        Metric(key, value, timestamp, step or 0, epoch or 0)
+        for key, value in metrics.items()
+    ]
+
     MlflowClient().log_batch(
-        run_id=run_id, metrics=[], params=params_arr, tags=[])
+        run_id=run_id, metrics=metrics_arr, params=params_arr, tags=tags_arr)
 
 
 def log_artifact(key, path, step=None):
@@ -455,161 +525,161 @@ def log_table(key, table, step=None):
         run_id, experiment_id, key, path, artifact_type='table', step=step)
 
 
-def log_bbox_prediction(key,
-                        image,
-                        bbox_pred,
-                        bbox_gt=None,
-                        bbox_type='pascal_voc',
-                        step=None):
-    """logs artifact for object detection.
-
-    Args:
-        key (str): name of the image
-        image (np.ndarray, PIL.Image): a numpy array of np.uint8 dtype or a
-                                    PIL.Image object
-        bbox_pred (np.ndarray, list): a list or a np.ndarray of dimension
-                                    (Nx4). All elemnts will be onverted to
-                                    int32.
-        bbox_gt (None, optional): a list or a np.ndarray of dimension (Nx4).
-                                    All elemnts will be onverted to int32
-        bbox_type (str, optional): can be one of 'yolo', 'pascal_voc', 'coco'
-                                indicating the format of bbox and prediction.
-        step (None, optional): integer indicating the step at which artifact
-                            was generated.
-
-    Raises:
-        MlflowException: Description
-    """
-
-    if bbox_type not in ['pascal_voc', 'coco', 'yolo']:
-        raise MlflowException(
-            f'bbox_type should be one-of "pascal_voc, coco, yolo" not \
-            {bbox_type}')
-
-    path = convert_to_imagefile(image)
-
-    bbox_pred = np.array(bbox_pred)
-    assert isinstance(
-        bbox_pred, np.ndarray) and bbox_pred.ndim == 2 and bbox_pred.shape[
-            1] == 4, f'bbox_pred should be numpy of dimension (Nx4), got \
-        {bbox_pred.shape}'
-
-    if bbox_type == 'coco':
-        bbox_pred = coco_to_voc_bbox(bbox_pred)
-        if bbox_gt:
-            bbox_gt = coco_to_voc_bbox(bbox_gt)
-    elif bbox_type == 'yolo':
-        bbox_pred = yolo_to_voc_bbox(path, bbox_pred)
-        if bbox_gt:
-            bbox_gt = yolo_to_voc_bbox(path, bbox_gt)
-
-    if bbox_gt is None:
-        bbox_gt = np.array([])
-
-    run = _get_or_start_run()
-    run_id = run.info.run_id
-    experiment_id = run.info.experiment_id
-
-    prediction_struct = Struct()
-    prediction_struct.update({'bbox': bbox_pred.tolist()})
-
-    ground_truth_struct = Struct()
-    ground_truth_struct.update({'bbox': bbox_gt.tolist()})
-
-    MlflowClient().log_artifact_lite(
-        run_id,
-        experiment_id,
-        key,
-        path,
-        prediction=prediction_struct,
-        ground_truth=ground_truth_struct,
-        artifact_type='image',
-        step=step,
-        tags={'image_type': 'bbox_prediction'})
+# def log_bbox_prediction(key,
+#                         image,
+#                         bbox_pred,
+#                         bbox_gt=None,
+#                         bbox_type='pascal_voc',
+#                         step=None):
+#     """logs artifact for object detection.
+#
+#     Args:
+#         key (str): name of the image
+#         image (np.ndarray, PIL.Image): a numpy array of np.uint8 dtype or a
+#                                     PIL.Image object
+#         bbox_pred (np.ndarray, list): a list or a np.ndarray of dimension
+#                                     (Nx4). All elemnts will be onverted to
+#                                     int32.
+#         bbox_gt (None, optional): a list or a np.ndarray of dimension (Nx4).
+#                                     All elemnts will be onverted to int32
+#         bbox_type (str, optional): can be one of 'yolo', 'pascal_voc', 'coco'
+#                                 indicating the format of bbox and prediction.
+#         step (None, optional): integer indicating the step at which artifact
+#                             was generated.
+#
+#     Raises:
+#         MlflowException: Description
+#     """
+#
+#     if bbox_type not in ['pascal_voc', 'coco', 'yolo']:
+#         raise MlflowException(
+#             f'bbox_type should be one-of "pascal_voc, coco, yolo" not \
+#             {bbox_type}')
+#
+#     path = convert_to_imagefile(image)
+#
+#     bbox_pred = np.array(bbox_pred)
+#     assert isinstance(
+#         bbox_pred, np.ndarray) and bbox_pred.ndim == 2 and bbox_pred.shape[
+#             1] == 4, f'bbox_pred should be numpy of dimension (Nx4), got \
+#         {bbox_pred.shape}'
+#
+#     if bbox_type == 'coco':
+#         bbox_pred = coco_to_voc_bbox(bbox_pred)
+#         if bbox_gt:
+#             bbox_gt = coco_to_voc_bbox(bbox_gt)
+#     elif bbox_type == 'yolo':
+#         bbox_pred = yolo_to_voc_bbox(path, bbox_pred)
+#         if bbox_gt:
+#             bbox_gt = yolo_to_voc_bbox(path, bbox_gt)
+#
+#     if bbox_gt is None:
+#         bbox_gt = np.array([])
+#
+#     run = _get_or_start_run()
+#     run_id = run.info.run_id
+#     experiment_id = run.info.experiment_id
+#
+#     prediction_struct = Struct()
+#     prediction_struct.update({'bbox': bbox_pred.tolist()})
+#
+#     ground_truth_struct = Struct()
+#     ground_truth_struct.update({'bbox': bbox_gt.tolist()})
+#
+#     MlflowClient().log_artifact_lite(
+#         run_id,
+#         experiment_id,
+#         key,
+#         path,
+#         prediction=prediction_struct,
+#         ground_truth=ground_truth_struct,
+#         artifact_type='image',
+#         step=step,
+#         tags={'image_type': 'bbox_prediction'})
 
 
-def log_mask_prediction(key,
-                        image,
-                        pred_mask,
-                        bbox_pred=[],
-                        mask_gt=None,
-                        bbox_gt=None,
-                        bbox_type='pascal_voc',
-                        step=None):
-    """logs artifact for instance/semantic segmentation.
-
-    Args:
-        key (str): name of the image
-        image (np.ndarray, PIL.Image): a numpy array of np.uint8 dtype or a
-                                    PIL.Image object
-        pred_mask (np.ndarray, PIL.Image): a numpy array of np.uint8 dtype or
-                                    a PIL.Image object
-        bbox_pred (None, optional): Description
-        mask_gt (np.ndarray, PIL.Image): a numpy array of np.uint8 dtype or a
-                                    PIL.Image object
-        bbox_gt (None, optional): a list or a np.ndarray of dimension (Nx4).
-                                    All elemnts will be onverted to int32
-        bbox_type (str, optional): Description
-        step (None, optional): integer indicating the step at which artifact
-                                    was generated
-    """
-    if bbox_type not in ['pascal_voc', 'coco', 'yolo']:
-        raise MlflowException(
-            f'bbox_type should be one-of "pascal_voc, coco, yolo" not \
-            {bbox_type}')
-
-    path = convert_to_imagefile(image)
-    pred_mask_path = convert_to_imagefile(pred_mask)
-
-    bbox_pred = np.array(bbox_pred)
-    if bbox_pred.size > 0:
-        print(bbox_pred.size)
-        assert isinstance(
-            bbox_pred, np.ndarray) and bbox_pred.ndim == 2 and bbox_pred.shape[
-                1] == 4, f'bbox_pred should be numpy of dimension (Nx4), got \
-            {bbox_pred.shape}'
-
-    if bbox_type == 'coco':
-        if bbox_pred.size > 0:
-            bbox_pred = coco_to_voc_bbox(bbox_pred)
-        if bbox_gt:
-            bbox_gt = coco_to_voc_bbox(bbox_gt)
-        else:
-            bbox_gt = np.array([])
-    else:
-        if bbox_pred.size > 0:
-            bbox_pred = yolo_to_voc_bbox(image, bbox_pred)
-        if bbox_gt:
-            bbox_gt = yolo_to_voc_bbox(image, bbox_gt)
-        else:
-            bbox_gt = np.array([])
-
-    run = _get_or_start_run()
-    run_id = run.info.run_id
-    experiment_id = run.info.experiment_id
-
-    prediction_struct = Struct()
-    prediction_struct.update({'bbox': bbox_pred.tolist()})
-
-    ground_truth_struct = Struct()
-    ground_truth_struct.update({'bbox': bbox_gt.tolist()})
-
-    MlflowClient().log_artifact_lite(
-        run_id,
-        experiment_id,
-        key,
-        path,
-        prediction=prediction_struct,
-        ground_truth=ground_truth_struct,
-        artifact_type='image',
-        step=step,
-        tags={'image_type': 'segmentation_mask'})
-
-    log_image(
-        key=key+'_mask',
-        image=pred_mask_path,
-        tags={'mask_parent': key},
-        step=step)
+# def log_mask_prediction(key,
+#                         image,
+#                         pred_mask,
+#                         bbox_pred=[],
+#                         mask_gt=None,
+#                         bbox_gt=None,
+#                         bbox_type='pascal_voc',
+#                         step=None):
+#     """logs artifact for instance/semantic segmentation.
+#
+#     Args:
+#         key (str): name of the image
+#         image (np.ndarray, PIL.Image): a numpy array of np.uint8 dtype or a
+#                                     PIL.Image object
+#         pred_mask (np.ndarray, PIL.Image): a numpy array of np.uint8 dtype or
+#                                     a PIL.Image object
+#         bbox_pred (None, optional): Description
+#         mask_gt (np.ndarray, PIL.Image): a numpy array of np.uint8 dtype or a
+#                                     PIL.Image object
+#         bbox_gt (None, optional): a list or a np.ndarray of dimension (Nx4).
+#                                     All elemnts will be onverted to int32
+#         bbox_type (str, optional): Description
+#         step (None, optional): integer indicating the step at which artifact
+#                                     was generated
+#     """
+#     if bbox_type not in ['pascal_voc', 'coco', 'yolo']:
+#         raise MlflowException(
+#             f'bbox_type should be one-of "pascal_voc, coco, yolo" not \
+#             {bbox_type}')
+#
+#     path = convert_to_imagefile(image)
+#     pred_mask_path = convert_to_imagefile(pred_mask)
+#
+#     bbox_pred = np.array(bbox_pred)
+#     if bbox_pred.size > 0:
+#         print(bbox_pred.size)
+#         assert isinstance(
+#             bbox_pred, np.ndarray) and bbox_pred.ndim == 2 and bbox_pred.shape[
+#                 1] == 4, f'bbox_pred should be numpy of dimension (Nx4), got \
+#             {bbox_pred.shape}'
+#
+#     if bbox_type == 'coco':
+#         if bbox_pred.size > 0:
+#             bbox_pred = coco_to_voc_bbox(bbox_pred)
+#         if bbox_gt:
+#             bbox_gt = coco_to_voc_bbox(bbox_gt)
+#         else:
+#             bbox_gt = np.array([])
+#     else:
+#         if bbox_pred.size > 0:
+#             bbox_pred = yolo_to_voc_bbox(image, bbox_pred)
+#         if bbox_gt:
+#             bbox_gt = yolo_to_voc_bbox(image, bbox_gt)
+#         else:
+#             bbox_gt = np.array([])
+#
+#     run = _get_or_start_run()
+#     run_id = run.info.run_id
+#     experiment_id = run.info.experiment_id
+#
+#     prediction_struct = Struct()
+#     prediction_struct.update({'bbox': bbox_pred.tolist()})
+#
+#     ground_truth_struct = Struct()
+#     ground_truth_struct.update({'bbox': bbox_gt.tolist()})
+#
+#     MlflowClient().log_artifact_lite(
+#         run_id,
+#         experiment_id,
+#         key,
+#         path,
+#         prediction=prediction_struct,
+#         ground_truth=ground_truth_struct,
+#         artifact_type='image',
+#         step=step,
+#         tags={'image_type': 'segmentation_mask'})
+#
+#     log_image(
+#         key=key+'_mask',
+#         image=pred_mask_path,
+#         tags={'mask_parent': key},
+#         step=step)
 
 
 def set_tags(tags):
@@ -626,29 +696,6 @@ def set_tags(tags):
         run_id=run_id, metrics=[], params=[], tags=tags_arr)
 
 
-def log_batch(metrics={}, params={}, tags={}, step=None):
-    """Summary.
-
-    Args:
-        metrics (dict, optional): Description
-        params (dict, optional): Description
-        tags (dict, optional): Description
-        step (None, optional): Description
-    """
-    run_id = _get_or_start_run().info.run_id
-    tags_arr = [RunTag(key, str(value)) for key, value in tags.items()]
-    params_arr = [Param(key, str(value)) for key, value in params.items()]
-
-    timestamp = int(time.time() * 1000)
-    metrics_arr = [
-        Metric(key, value, timestamp, step or 0)
-        for key, value in metrics.items()
-    ]
-
-    MlflowClient().log_batch(
-        run_id=run_id, metrics=metrics_arr, params=params_arr, tags=tags_arr)
-
-
 def _record_logged_model(mlflow_model):
     """Summary.
 
@@ -858,18 +905,6 @@ def _get_paginated_runs(experiment_ids, filter_string, run_view_type,
     return all_runs
 
 
-def _get_or_start_run():
-    """Summary.
-
-    Returns:
-        TYPE: Description
-    """
-    if len(_active_run_stack) > 0:
-        return _active_run_stack[-1]
-
-    return start_run()
-
-
 def _get_experiment_id_from_env():
     """Summary.
 
